#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Mar 21 11:53:11 2020

@author: farid
"""
import numpy as np
import pandas as pd
import corner
import scipy
import scipy.stats
import matplotlib
import matplotlib.pyplot as plt

class Distribution(object):
    
    def __init__(self,dist_names_list = []):
        self.dist_names = dist_names_list
        self.dist_results = []
        self.params = {}
        
        self.DistributionName = ""
        self.PValue = 0
        self.Param = None
        
        self.isFitted = False
        
        
    def Fit(self, y):
        self.dist_results = []
        self.params = {}
        for dist_name in self.dist_names:
            dist = getattr(scipy.stats, dist_name)
            
            param = dist.fit(y) if dist_name != 'lognorm' else dist.fit(np.exp(y),floc=0)
            # param = dist.fit(y)
            
            self.params[dist_name] = param
            #Applying the Kolmogorov-Smirnov test
            D, p = scipy.stats.kstest(y, dist_name, args=param);
            self.dist_results.append((dist_name,p))
        #select the best fitted distribution
        sel_dist,p = (max(self.dist_results,key=lambda item:item[1]))
        #store the name of the best fit and its p value
        self.DistributionName = sel_dist
        self.PValue = p
        self.Param = self.params[self.DistributionName]
        self.isFitted = True
        return self.DistributionName,self.PValue
    
    def Random(self, n = 1):
        if self.isFitted:
            dist_name = self.DistributionName
            param = self.params[dist_name]
        
            #initiate the scipy distribution
            dist = getattr(scipy.stats, dist_name)
            return dist.rvs(*param[:-2], loc=param[-2], scale=param[-1], size=n)
        else:
            raise ValueError('Must first run the Fit method.')
            
    def Plot(self,data, fitted=None):
        fig = plt.figure()
        fitted = self.Random(n=len(data)) if fitted is None else fitted
        plt.hist(fitted, bins=50, alpha=0.5, label='Fitted')
        plt.hist(data, bins=50, alpha=0.5, label='Data')
        plt.legend(loc='upper right')





if __name__ == "__main__":
    
    import chaospy
    mcParametersets = pd.read_csv('./../InputParameters.csv')
    
    
    print("Summary of parameters:\n", mcParametersets.describe())
    
    Prior = mcParametersets.to_numpy()
    figPrior = corner.corner(Prior, labels=["Permeability","Porosity","LeakWellPermeability"],
                           show_titles=True, title_kwargs={"fontsize": 12})
    figPrior.set_size_inches((10,10)) 
    
    Dist = Distribution(['lognorm', 'norm','uniform', 'expon'])#['uniform', 'lognorm', 'expon', 'norm'])
    
    for parName in mcParametersets.columns:
        
        DistributionName, PValue = Dist.Fit(mcParametersets[parName])
        print("\nFitted distribution of paramter:", parName)
        print("type:", DistributionName)    
        print("Dist params:", Dist.Param)
        
        if DistributionName =='norm':
            Params = Dist.Param
            Mu = np.log(Params[0]**2 / np.sqrt(Params[0]**2 + Params[1]**2))
            Sigma  = np.sqrt(np.log(1 + Params[1]**2 / Params[0]**2))
            dist = chaospy.LogNormal(mu=Mu,sigma=Sigma)
            Dist.Plot(data=mcParametersets[parName],fitted=dist.sample(1260))
        else:
            Dist.Plot(mcParametersets[parName])
        # Dist.Plot(mcParametersets[parName])
        # if parName == "LeakWellPermeability":
        #     Params = (1.0e-12, 4.0e-13) #(-27.7, 0.4)
        #     Mu = np.log(Params[0]**2 / np.sqrt(Params[0]**2 + Params[1]**2))
        #     Sigma  = np.sqrt(np.log(1 + Params[1]**2 / Params[0]**2))
        #     dist = chaospy.LogNormal(mu=Mu,sigma=Sigma)
        #     # dist = chaospy.LogNormal(mu=0,sigma=Sigma, scale=Params[0])
        #     Dist.Plot(data=mcParametersets[parName],fitted=dist.sample(1260))
        # else:
        #     Dist.Plot(mcParametersets[parName])